package com.qmth.themis.backend.api;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.themis.backend.config.DictionaryConfig;
import com.qmth.themis.backend.util.ServletUtil;
import com.qmth.themis.business.config.SystemConfig;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBAttachment;
import com.qmth.themis.business.entity.TBPrivilege;
import com.qmth.themis.business.entity.TBTaskHistory;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.enums.DownloadFileEnum;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.service.TBAttachmentService;
import com.qmth.themis.business.service.TBPrivilegeService;
import com.qmth.themis.business.service.TBTaskHistoryService;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 系统信息 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/27
 */
@Api(tags = "系统信息Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/sys")
public class SysController {

    @Resource
    TBPrivilegeService tbPrivilegeService;

    @Resource
    TBAttachmentService tbAttachmentService;

    @Resource
    TBTaskHistoryService tbTaskHistoryService;

    @Resource
    DictionaryConfig dictionaryConfig;

    @Resource
    SystemConfig systemConfig;

    @ApiOperation(value = "菜单查询接口")
    @RequestMapping(value = "/getMenu", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "菜单信息", response = TBPrivilege.class)})
    public Result getMenu() {
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
        if (Objects.isNull(tbUser)) {
            throw new BusinessException(ExceptionResultEnum.LOGIN_NO);
        }
        List<TBPrivilege> tbPrivilegeList = null;
        if (Objects.nonNull(tbUser.getRemark()) && Objects.equals(tbUser.getRemark(), RoleEnum.SUPER_ADMIN.getCode())) {
            QueryWrapper<TBPrivilege> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(TBPrivilege::getType, SystemConstant.MENU).orderByAsc(TBPrivilege::getParentId, TBPrivilege::getSequence);
            tbPrivilegeList = tbPrivilegeService.list(wrapper);
        } else {
            tbPrivilegeList = tbPrivilegeService.getMenu(tbUser.getId());
        }
        Map map = new HashMap();
        map.put(SystemConstant.RECORDS, tbPrivilegeList);
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "获取环境接口")
    @RequestMapping(value = "/env", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "环境信息", response = Result.class)})
    public Result env(@ApiParam(value = "机构id", required = true) @RequestParam Long orgId) {
        if (Objects.isNull(orgId) || Objects.equals(orgId, "")) {
            throw new BusinessException(ExceptionResultEnum.ORG_ID_IS_NULL);
        }
//        SchoolDto schoolDto = (SchoolDto) redisTemplate.opsForValue().get(SystemConstant.SCHOOL_CACHE + orgId);
//        Map map = new HashMap();
//        map.put(SystemConstant.ENV_FILEHOST, schoolDto.getFileHost());
//        return ResultUtil.ok(map);
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "上传文件接口")
    @RequestMapping(value = "/file/upload", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "{\"id\":0}", response = Result.class)})
    public Result fileUpload(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file, @ApiParam(value = "上传文件类型", required = true) @RequestParam Integer type) throws IOException {
        if (Objects.isNull(file) || Objects.equals(file, "")) {
            throw new BusinessException(ExceptionResultEnum.ATTACHMENT_IS_NULL);
        }
        Map<String, Object> mapParameter = systemConfig.getOssEnv(type);
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
        TBAttachment tbAttachment = tbAttachmentService.saveAttachment(file, ServletUtil.getRequestMd5(), ServletUtil.getRequestPath(), mapParameter, tbUser.getOrgId(), tbUser.getId());
        if (Objects.isNull(tbAttachment)) {
            throw new BusinessException(ExceptionResultEnum.ATTACHMENT_ERROR);
        }
        Map map = new HashMap();
        map.put(SystemConstant.ID, tbAttachment.getId());
        map.put("name", tbAttachment.getName());
        map.put(SystemConstant.TYPE, tbAttachment.getType());
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "文件下载接口")
    @RequestMapping(value = "/file/download", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"url\":string}", response = Result.class)})
    public Result fileDownload(@ApiParam(value = "任务id", required = true) @RequestParam Long id, @ApiParam(value = "下载文件类型", required = true) @RequestParam Integer type) {
        if (Objects.isNull(id) || Objects.equals(id, "")) {
            throw new BusinessException(ExceptionResultEnum.TASK_ID_IS_NULL);
        }
        if (Objects.isNull(type) || Objects.equals(type, "")) {
            throw new BusinessException(ExceptionResultEnum.DOWNLOAD_FILE_TYPE_IS_NULL);
        }
        if (Objects.isNull(DownloadFileEnum.convertToName(type))) {
            throw new BusinessException(ExceptionResultEnum.DOWNLOAD_FILE_TYPE_ERROR);
        }
        TBTaskHistory tbTaskHistory = tbTaskHistoryService.getById(id);
        if (Objects.isNull(tbTaskHistory)) {
            throw new BusinessException(ExceptionResultEnum.TASK_NO);
        }
        boolean oss = dictionaryConfig.sysDomain().isOss();
        JSONObject jsonObject = null;
        switch (type) {
            case 0:
                jsonObject = JSONObject.parseObject(tbTaskHistory.getImportFilePath());
                break;
            case 1:
                jsonObject = JSONObject.parseObject(tbTaskHistory.getErrorFilePath());
                break;
            default:
                jsonObject = JSONObject.parseObject(tbTaskHistory.getResultFilePath());
                break;
        }
        if (Objects.isNull(jsonObject) || Objects.isNull(jsonObject.get("path"))) {
            throw new BusinessException("下载文件地址不存在");
        }
        String filePath = String.valueOf(jsonObject.get("path"));
        String url = null;
        if (oss) {
            url = dictionaryConfig.aliYunOssDomain().getUrl() + File.separator + filePath;
        } else {
            url = "http://" + dictionaryConfig.sysDomain().getFileHost() + File.separator + filePath;
        }
        Map map = new HashMap();
        map.put("url", url);
        return ResultUtil.ok(map);
    }
}
